
#include "ValueExpressionDriver.H"
#include "ValueExpressionParser.tab.hh"
#include <Random.H>
#include <wallDist.H>

ValueExpressionDriver::ValueExpressionDriver (const Foam::string &time,const Foam::fvMesh &mesh)
    : time(time),
      mesh(mesh),
      result(NULL),
      vresult(NULL),
      typ(NO_TYPE),
      trace_scanning (false), 
      trace_parsing (false)
{
}

ValueExpressionDriver::~ValueExpressionDriver ()
{
    if(result) {
        delete result;
    }
    if(vresult) {
        delete vresult;
    }
}

void ValueExpressionDriver::setScalarResult(Foam::volScalarField *r) {
    result=r;
    typ=SCALAR_TYPE;
}

void ValueExpressionDriver::setLogicalResult(Foam::volScalarField *r) {
    result=r;
    typ=LOGICAL_TYPE;
}

void ValueExpressionDriver::setVectorResult(Foam::volVectorField *r) {
    vresult=r;
    typ=VECTOR_TYPE;
}

void ValueExpressionDriver::parse (const std::string &f)
{
    content = f;
    scan_begin ();
    yy::ValueExpressionParser parser (*this);
    parser.set_debug_level (trace_parsing);
    parser.parse ();
    scan_end ();
}

void ValueExpressionDriver::error (const yy::location& l, const std::string& m)
{
    std::ostringstream buff;
    buff << l;
    std::string place="";
    for(unsigned int i=0;i<l.begin.column;i++) {
        place+=" ";
    }
    for(unsigned int i=l.begin.column;i<l.end.column;i++) {
        place+="^";
    }
    for(unsigned int i=l.end.column;i<content.size();i++) {
        place+=" ";
    }

    Foam::FatalErrorIn("parsingValue")
        //        << Foam::args.executable()
        << " Parser Error at " << buff.str() << " :"  << m << Foam::endl
            << content << Foam::endl << place
            << Foam::exit(Foam::FatalError);
    //    Foam::Info << buff.str() << ": " << m << Foam::endl;
}

void ValueExpressionDriver::error (const std::string& m)
{
    Foam::FatalErrorIn("parsingValue")
        //        << Foam::args.executable()
            << " Parser Error: " << m
            << Foam::exit(Foam::FatalError);
}
bool ValueExpressionDriver::isScalar(const Foam::string &name)
{
    if(getTypeOfField(name)=="volScalarField") {
        return true;
    } else {
        return false;
    }
}

bool ValueExpressionDriver::isVector(const Foam::string &name)
{
    if(getTypeOfField(name)=="volVectorField") {
        return true;
    } else {
        return false;
    }
}

Foam::string ValueExpressionDriver::getTypeOfField(const Foam::string &name)
{
    Foam::IOobject f 
        (
            name,
            time,
            mesh,
            Foam::IOobject::MUST_READ,
            Foam::IOobject::NO_WRITE
        );
    f.headerOk();

    return f.headerClassName();
}

Foam::volScalarField *ValueExpressionDriver::getScalarField(const Foam::string &name)
{
    Foam::dimensionSet nullDim(0,0,0,0,0);

    Foam::volScalarField *f=new Foam::volScalarField(
        Foam::IOobject
        (
            name,
            time,
            mesh,
            Foam::IOobject::MUST_READ,
            Foam::IOobject::NO_WRITE
        ),
        mesh
    );
    f->dimensions().reset(nullDim);

    return f;
}

Foam::volScalarField *ValueExpressionDriver::makeRandomField()
{
    Foam::volScalarField *f=makeScalarField(0.);
    Foam::Random rand(65);

    forAll(*f,cellI) {
        (*f)[cellI]=rand.scalar01();
    }

    return f;
}

Foam::volScalarField *ValueExpressionDriver::makeGaussRandomField()
{
    Foam::volScalarField *f=makeScalarField(0.);
    Foam::Random rand(65);

    forAll(*f,cellI) {
        (*f)[cellI]=rand.GaussNormal();
    }

    return f;
}

Foam::volVectorField *ValueExpressionDriver::makePositionField()
{
    Foam::dimensionSet nullDim(0,0,0,0,0);
    Foam::volVectorField *f=new Foam::volVectorField(
        Foam::IOobject
        (
            "pos",
            time,
            mesh,
            Foam::IOobject::NO_READ,
            Foam::IOobject::NO_WRITE
        ),
        mesh,
        Foam::vector(0,0,0)
    );
    f->dimensions().reset(mesh.C().dimensions());
    *f=mesh.C();
    f->dimensions().reset(nullDim);
    return f;
}

Foam::volScalarField *ValueExpressionDriver::makeDistanceField()
{
    Foam::dimensionSet nullDim(0,0,0,0,0);
    Foam::volScalarField *f=new Foam::volScalarField(
        Foam::IOobject
        (
            "dist",
            time,
            mesh,
            Foam::IOobject::NO_READ,
            Foam::IOobject::NO_WRITE
        ),
        mesh,
        0
    );
    f->dimensions().reset(mesh.C().dimensions());
    Foam::wallDist dist(mesh);
    *f=dist;
    f->dimensions().reset(nullDim);
    return f;

}

Foam::volVectorField *ValueExpressionDriver::getVectorField(const Foam::string &name)
{
    Foam::dimensionSet nullDim(0,0,0,0,0);
    Foam::volVectorField *f=new Foam::volVectorField(
        Foam::IOobject
        (
            name,
            time,
            mesh,
            Foam::IOobject::MUST_READ,
            Foam::IOobject::NO_WRITE
        ),
        mesh
    );

    f->dimensions().reset(nullDim);
    return f;
}

Foam::volScalarField *ValueExpressionDriver::makeScalarField(const Foam::scalar &val)
{
    std::ostringstream buff;
    buff << "constantScalar" << val;

    Foam::volScalarField *f=new Foam::volScalarField(
        Foam::IOobject
        (
            buff.str(),
            time,
            mesh,
            Foam::IOobject::NO_READ,
            Foam::IOobject::NO_WRITE
        ),
        mesh,
        val
    );

    return f;
}

Foam::volVectorField *ValueExpressionDriver::makeVectorField(const Foam::vector &vec)
{
    std::ostringstream buff;
    buff << "constantVector" << vec.x() << "_" << vec.y() << "_" << vec.z() ;

    Foam::volVectorField *f=new Foam::volVectorField(
        Foam::IOobject
        (
            buff.str(),
            time,
            mesh,
            Foam::IOobject::NO_READ,
            Foam::IOobject::NO_WRITE
        ),
        mesh,
        vec
    );

    return f;
}

Foam::volVectorField *ValueExpressionDriver::makeVectorField(Foam::volScalarField *x,Foam::volScalarField *y,Foam::volScalarField *z)
{
    Foam::volVectorField *f=makeVectorField(Foam::vector(0,0,0));

    forAll(*f,cellI) {
        (*f)[cellI]=Foam::vector((*x)[cellI],(*y)[cellI],(*z)[cellI]);
    }

    return f;
}

Foam::volScalarField *ValueExpressionDriver::doLogicalNot(Foam::volScalarField *a)
{
    Foam::volScalarField *res=makeScalarField(0.);

    forAll(*res,cellI) {
        bool av= (*a)[cellI] ? true : false;

        if( !av ) {
            (*res)[cellI]=1;
        } else {
            (*res)[cellI]=0;
        }
    }

    return res;
}

