
#ifndef VALUE_EXPRESSION_DRIVER_H
#define VALUE_EXPRESSION_DRIVER_H

#include <string.H>
#include <scalar.H>

#include <volFields.H>

union YYSTYPE;

namespace yy {
    class location;
    class ValueExpressionParser;
}

class ValueExpressionDriver;

#define YY_DECL int velex(YYSTYPE* yylval,yy::location* yylloc,ValueExpressionDriver &driver)

YY_DECL;

class ValueExpressionDriver {
public:
    enum ExpressionType{
        NO_TYPE,
        SCALAR_TYPE,
        VECTOR_TYPE,
        LOGICAL_TYPE
    };

private:
    Foam::string getTypeOfField(const Foam::string &name);

    const Foam::string &time;
    const Foam::fvMesh &mesh;

    Foam::volScalarField *result;
    Foam::volVectorField *vresult;

    ExpressionType typ;

public:
    ValueExpressionDriver(const Foam::string &time,const Foam::fvMesh &mesh);
    virtual ~ValueExpressionDriver();

    // Check the result type
    bool resultIsVector() { return typ==VECTOR_TYPE; }
    bool resultIsScalar() { return typ==SCALAR_TYPE; }
    bool resultIsLogical() { return typ==LOGICAL_TYPE; }

    // Setting the result
    void setScalarResult(Foam::volScalarField *);
    void setLogicalResult(Foam::volScalarField *);
    void setVectorResult(Foam::volVectorField *);

    // Getting the results
    Foam::volScalarField *getScalar() { return result; }
    Foam::volVectorField *getVector() { return vresult; }

    // Handling the scanner.
    void scan_begin ();
    void scan_end ();
    bool trace_scanning;

    // Handling the parser.
    void parse (const std::string& f);
    Foam::string content;
    bool trace_parsing;

    // Error handling.
    void error (const yy::location& l, const std::string& m);
    void error (const std::string& m);

    // checking for variables
    bool isScalar(const Foam::string &name);
    bool isVector(const Foam::string &name);

    // returning fields
    Foam::volScalarField *getScalarField(const Foam::string &name);
    Foam::volVectorField *getVectorField(const Foam::string &name);

    // make constant fields
    Foam::volScalarField *makeScalarField(const Foam::scalar &val);
    Foam::volVectorField *makeVectorField(const Foam::vector &vec);
    Foam::volVectorField *makeVectorField(Foam::volScalarField *x,Foam::volScalarField *y,Foam::volScalarField *z);

    // make special fields
    Foam::volVectorField *makePositionField();
    Foam::volScalarField *makeDistanceField();
    Foam::volScalarField *makeRandomField();
    Foam::volScalarField *makeGaussRandomField();
    
    // Logical expressions
    template <class Op>
    Foam::volScalarField *doCompare(Foam::volScalarField *a,Op op,Foam::volScalarField *b);

    template <class Op>
    Foam::volScalarField *doLogicalOp(Foam::volScalarField *a,Op op,Foam::volScalarField *b);

    Foam::volScalarField *doLogicalNot(Foam::volScalarField *a);
    
    template <class T>
    T * doConditional(Foam::volScalarField *l,T* yes,T* no,T* result);
};

#endif
