/*---------------------------------------------------------------------------*\
 =========                   |
 \\      /   F ield          | OpenFOAM: The Open Source CFD Toolbox
  \\    /    O peration      |
   \\  /     A nd            | Copyright (C) 1991-2005 OpenCFD Ltd.
    \\/      M anipulation   |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    funkySetFields

Description

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"

#include "ValueExpressionDriver.H"

template<class T>
void setField(const string &name,const fvMesh &mesh,const string &time,T *result,volScalarField *cond)
{
    T tmp
        (
            IOobject  
            (
                name,
                time,
                mesh,
                IOobject::MUST_READ,
                IOobject::WRITE
            ),
            mesh
        );

    label setCells=0;

    forAll(tmp,cellI) {
        if((*cond)[cellI]!=0) {
            tmp[cellI]=(*result)[cellI];
            setCells++;
        }
    }

    Info << " Setting " << setCells << " of " << tmp.size() << " cells" << endl;

    Info << " Writing to " << name << endl;

    tmp.write();
}

void doAnExpression(const fvMesh &mesh,const word &field,const string &expression,const string &condition,const string &time,bool doDebug)
{
    IOobject f 
        (
            field,
            time,
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        );
    f.headerOk();

    bool isScalar=false;

    word classN=f.headerClassName();
    if(classN=="volScalarField") {
        isScalar=true;
    } else if (classN!="volVectorField") {
        FatalErrorIn("doAnExpression()")
            //            << args.executable()
            << " unsupported type " << classN << " of field " << field << " or not existing at time " << time
            << exit(FatalError);
    }

    Info << " Putting " << expression << " into field " << field << " at T = " << time << " if condition " << condition << " is true\n" << endl;

    ValueExpressionDriver driver(time,mesh);
    ValueExpressionDriver ldriver(time,mesh);

    if (doDebug) {
        Info << "Parsing expression: " << expression << "\nand condition " << condition << "\n" << endl; 
        driver.trace_parsing=true;
        driver.trace_scanning=true;
    }

    ldriver.parse(condition);
    if(!ldriver.resultIsLogical()) {
        FatalErrorIn("doAnExpression()")
            //            << args.executable()
                << " condition: " << condition << " does not evaluate to a logical expression" 
            << exit(FatalError);
    }

    driver.parse(expression);
    if(driver.resultIsVector()==isScalar) {
        FatalErrorIn("doAnExpression()")
            //            << args.executable()
                << " inconsistent types: " << field << " is  " << (isScalar ? "scalar" : "vector" ) << " while the expression evaluates to a " << (!driver.resultIsVector() ? "scalar" : "vector" )
            << exit(FatalError);
    } else {
        if(isScalar) {
            setField(field,mesh,time,driver.getScalar(),ldriver.getScalar());
        } else {
            setField(field,mesh,time,driver.getVector(),ldriver.getScalar());
        }
    }
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Main program:

int main(int argc, char *argv[])
{

#   include "addTimeOption.H"

    argList::validOptions.insert("field","field to overwrite");
    argList::validOptions.insert("expression","expression to write");
    argList::validOptions.insert("condition","logical condition");
    argList::validOptions.insert("debugParser","");

#   include "setRootCase.H"

    if (!args.options().found("time")) {
        FatalErrorIn("main()")
            << args.executable()
            << ": time option is required"
            << exit(FatalError);
    }

#   include "createTime.H"
    instantList Times = runTime.times();
#   include "checkTimeOptions.H"
    runTime.setTime(Times[startTime],startTime);
#   include "createMesh.H"

    if(args.options().found("field")) {
        Info << " Using command-line options\n" << endl;

        word field=args.options()["field"];
        
        string expression=args.options()["expression"];

        string condition="true";
        if (args.options().found("condition")) {
            condition=args.options()["condition"];
        }

        doAnExpression(mesh,field,expression,condition,runTime.timeName(),args.options().found("debugParser"));        
    } else {
        Info << " Using funkySetFieldsDict \n" << endl;
        
        IOdictionary funkyDict
            (
                IOobject
                (
                    "funkySetFieldsDict",
                    runTime.system(),
                    mesh,
                    IOobject::MUST_READ,
                    IOobject::NO_WRITE
                )
            );

        PtrList<entry> parts=funkyDict.lookup("expressions");

        forAll(parts,partI) {
            const dictionary &part=parts[partI].dict();

            Info << "Part: " << parts[partI].keyword() << endl;

            word field=part["field"];

            string expression=part["expression"];

            string condition="true";

            if (part.found("condition")) {
                condition=part["condition"];
            }

            doAnExpression(mesh,field,expression,condition,runTime.timeName(),args.options().found("debugParser"));        
        }
    }
    
    Info << "End\n" << endl;

    return 0;
}


// ************************************************************************* //
